"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveEnvironments = exports.EnvironmentService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const fs = require("fs");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const SSH2Promise = require("ssh2-promise");
let Client = require('ftp');
const config_1 = require("../config/config");
const aws_s3_implementation_1 = require("../implementation/aws-s3.implementation");
const aws_model_1 = require("../models/aws.model");
let logger = log4js.getLogger('ServerScripts');
function EnvironmentService(app) {
    app.post('/api/environment/get_environments/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('[EnvironmentService::get_environments] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        let search_integrated = req.body.includeIntegrated !== undefined ? req.body.includeIntegrated : true;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetEnvironments(vcc, search_criteria, search_expression, search_integrated);
        }).then(resultEnvironments => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultEnvironments });
        }).catch(err => {
            let environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.get('/api/environment/get_environment/:environment_id', (req, res, next) => {
        logger.info('[EnvironmentService::get_environment] Received ' + req.method + ' on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let environmentId = req.params.environment_id;
        let environment = null;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!environmentId)
                throw 'ENVIRONMENT_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetEnvironment(vcc, environmentId);
        }).
            then(env => {
            environment = env;
            return mssql_service_1.MSSql.GetEnvironmentsUsage(vcc, environmentId);
        }).then(usages => {
            if (!environment)
                throw 'ENVIRONMENT_SERVER_ERROR_NOT_FOUND';
            if (res.headersSent)
                return;
            environment.dbPassword = '******';
            environment.pbxPassword = '******';
            environment.inUse = (usages.plans.length > 0 || usages.process.length > 0);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: environment, usages });
        }).catch(err => {
            let environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EnvironmentService::get_environment] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.get('/api/environment/get_environment_type_by_name/:environment_id', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let environmentId = req.params.environment_id;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!environmentId)
                throw 'ENVIRONMENT_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetEnvironmentTypeByName(vcc, environmentId);
        }).then((resultEnvironmentType) => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultEnvironmentType });
        }).catch(err => {
            let environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/environment/create_environment', (req, res, next) => {
        logger.info('[EnvironmentService::create_environment] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let newEnvironment = (req.body || {});
        newEnvironment.bucket = newEnvironment.bucket ? newEnvironment.bucket.trim() : null;
        newEnvironment.apiUser = newEnvironment.apiUser.trim();
        newEnvironment.apiPassword = newEnvironment.apiPassword.trim();
        newEnvironment.apiHost = newEnvironment.apiHost.trim();
        newEnvironment.apiPort = newEnvironment.apiPort;
        newEnvironment.createdDate = nowUtc;
        newEnvironment.createdByUserId = userId;
        newEnvironment.lastModifiedDate = nowUtc;
        newEnvironment.lastModifiedByUserId = userId;
        newEnvironment.dbDisabled = (newEnvironment.dbDisabled ? newEnvironment.dbDisabled : false);
        logger.info('[EnvironmentService::create_environment] Params: ', newEnvironment);
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetEnvironmentByName(vcc, newEnvironment.name);
        }).then(resultEnvironment => {
            if (resultEnvironment)
                throw 'ENVIRONMENT_SERVER_ERROR_EXISTS';
            return SaveEnvironment(vcc, newEnvironment);
        }).then(resultSaveEnvironment => {
            return Promise.resolve(resultSaveEnvironment);
        }).then(resultSaveEnvironment => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSaveEnvironment ? 'SERVER_SUCCESS_MESSAGE' : 'ENVIRONMENT_SERVER_ERROR_ON_SAVE'), data: resultSaveEnvironment });
        }).catch(err => {
            let environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EnvironmentService::create_environment] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/environment/save_environment', (req, res, next) => {
        logger.info('[EnvironmentService::save_environment] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let environment = (req.body || {});
        environment.apiUser = environment.apiUser.trim();
        environment.apiPassword = environment.apiPassword.trim();
        environment.apiHost = environment.apiHost.trim();
        environment.apiPort = environment.apiPort;
        environment.createdDate = null;
        environment.createdByUserId = null;
        environment.lastModifiedDate = nowUtc;
        environment.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(resultDeleteEnvironment => {
            if (!resultDeleteEnvironment)
                throw 'SERVER_ERROR_UNKNOWN_MESSAGE';
            return SaveEnvironment(vcc, environment);
        }).then(resultSaveEnvironment => {
            if (!resultSaveEnvironment)
                return false;
            return Promise.resolve(true);
        }).then(resultSaveEnvironment => {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveEnvironment, description: (resultSaveEnvironment ? 'SERVER_SUCCESS_MESSAGE' : 'ENVIRONMENT_SERVER_ERROR_ON_SAVE'), data: resultSaveEnvironment });
        }).catch(err => {
            let environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EnvironmentService::save_environment] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/environment/delete_environment', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let environment = req.body.environment;
        let environmentId = environment.name;
        logger.info(`[EnvironmentService::delete_environment] Deleting environment ${environmentId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!environment)
                throw 'ENVIRONMENT_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetEnvironmentsUsage(vcc, environmentId);
        })
            .then(usages => {
            if (usages.plans.length > 0 || usages.process.length > 0) {
                throw 'ENVIRONMENT_IS_USED';
            }
            return mssql_service_1.MSSql.DeleteEnvironment(vcc, environmentId);
        }).then(resultDeleteEnvironment => {
            if (!resultDeleteEnvironment)
                throw 'ENVIRONMENT_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(resultDeleteEnvironment => {
            if (res.headersSent)
                return;
            if (resultDeleteEnvironment)
                logger.info('Ambiente \'' + environmentId + '\' eliminado correctamente');
            res.json({ status: resultDeleteEnvironment, description: (resultDeleteEnvironment ? 'SERVER_SUCCESS_MESSAGE' : 'ENVIRONMENT_SERVER_ERROR_ON_DELETE'), data: resultDeleteEnvironment });
        }).catch(err => {
            let environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/environment/delete_environments', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let environments = req.body.environments || [];
        let environmentIds = environments.map(e => e.name);
        logger.info(`[EnvironmentService::delete_environments] Deleting environments ${environmentIds.join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!environments)
                throw 'ENVIRONMENT_SERVER_ERROR_MISSING_IDS';
            return mssql_service_1.MSSql.GetEnvironmentsUsage(vcc, environmentIds.toString());
        })
            .then(usages => {
            if (usages.plans.length > 0 || usages.process.length > 0) {
                throw 'ENVIRONMENTS_IN_USE';
            }
            let promiseArray = [];
            environmentIds.forEach(environment => {
                promiseArray.push(mssql_service_1.MSSql.DeleteEnvironment(vcc, environment));
            });
            return Promise.all(promiseArray);
        }).then(resultDeleteEnvironments => {
            let result;
            if (resultDeleteEnvironments.filter(result => { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Ambientes eliminados correctamente');
            }
            return result;
        }).then(resultDeleteEnvironments => {
            if (res.headersSent)
                return;
            let description = (resultDeleteEnvironments ? 'SERVER_SUCCESS_MESSAGE' : 'ENVIRONMENT_SERVER_ERROR_ON_DELETE');
            res.json({
                status: resultDeleteEnvironments,
                description,
                data: resultDeleteEnvironments
            });
        }).catch(err => {
            let environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
    app.post('/api/environment/test_environment', (req, res) => {
        let params = {
            host: req.body.apiHost,
            port: req.body.apiPort,
            password: req.body.apiPassword,
        };
        switch (req.body.type) {
            case 'sftp':
            case 'i-sftp':
                TestSFTP(Object.assign(params, {
                    username: req.body.apiUser,
                    readyTimeout: config_1.FTP_CONNECTION_TIMEOUT
                }), req.body.ftpDirectory).then(result => {
                    res.json(result);
                });
                break;
            case 'ftp':
            case 'i-ftp':
                TestFTP(Object.assign(params, {
                    user: req.body.apiUser,
                    connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
                    pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT,
                }), req.body.ftpDirectory).then(result => {
                    res.json(result);
                }).catch(err => {
                    logger.error('[EnvironmentService::test_environment] Error on \'' + req.path + '\': ' + err);
                    res.json({ status: false, code: err });
                });
                break;
            case 's3':
            case 'i-s3':
                TestS3({
                    region: req.body.apiHost,
                    accessKeyId: req.body.apiUser,
                    secretAccessKey: req.body.apiPassword,
                    bucket: req.body.bucket,
                    path: req.body.ftpDirectory
                })
                    .then(result => {
                    res.json(result);
                })
                    .catch(err => {
                    logger.error('[EnvironmentService::test_environment] Error on \'' + req.path + '\': ' + err);
                    res.json({ status: false, code: err });
                });
                break;
            default:
                break;
        }
    });
    function TestSFTP(config, dir) {
        if (config.host === 's-b119f51bb73843339.server.transfer.eu-west-1.amazon') {
            config.host = 's-b119f51bb73843339.server.transfer.eu-west-1.amazonaws.com';
            config.privateKey = fs.readFileSync(`${config_1.PATH_FOLDER_SSL}/NewPrivatekey`, 'utf-8');
        }
        let ssh2 = new SSH2Promise(config);
        let response = { status: false };
        return new Promise((resolve, reject) => {
            ssh2.connect().then(() => {
                let sftp = ssh2.sftp();
                sftp.readdir(dir).then(result => {
                    response.status = result ? true : false;
                    response.code = 'SERVER_SUCCESS_SFTP_CONNECTION';
                    response.result = result;
                    sftp.close();
                    resolve(response);
                }).catch(err => {
                    logger.error('[EnvironmentService::test_environment] Error on TestSFTP ', err.toString());
                    response.code = 'SERVER_ERROR_SFTP_PATH_NOT_FOUND';
                    resolve(response);
                });
            }).catch(err => {
                logger.error('[EnvironmentService::test_environment] Error on TestSFTP ', err.toString());
                if (err.level == "client-authentication") {
                    response.code = 'SERVER_ERROR_BAD_USER_OR_PASSWORD';
                }
                else if (err.level == "client-socket") {
                    response.code = 'SERVER_ERROR_BAD_IP';
                }
                else {
                    response.code = 'SERVER_ERROR_SFTP_NOT_CLIENT';
                }
                resolve(response);
            });
        });
    }
    function TestFTPDir(client, dir) {
        return new Promise((resolve, reject) => {
            client.list(dir, (err, list) => {
                if (err) {
                    logger.error('[EnvironmentService::test_environment] Error on TestFTPDir ', err.toString());
                    reject("SERVER_ERROR_FTP_PATH_NOT_FOUND");
                }
                else {
                    resolve({ status: true, list, code: 'SERVER_SUCCESS_FTP_CONNECTION' });
                }
            });
        });
    }
    function TestFTP(config, dir) {
        return new Promise(async (resolve, reject) => {
            let clientFtp = new Client();
            try {
                await clientFtp.connect(config);
                if (clientFtp) {
                    clientFtp.on('ready', async () => {
                        try {
                            resolve(await TestFTPDir(clientFtp, dir));
                        }
                        catch (error) {
                            reject(error);
                        }
                    });
                    clientFtp.on('end', () => {
                        reject('SERVER_ERROR_FTP_CONNECTION_ENDED');
                    });
                    clientFtp.on('close', hadError => {
                        reject('SERVER_ERROR_FTP_CONNECTION_REFUSED');
                    });
                    clientFtp.on('error', err => {
                        logger.error('[EnvironmentService::test_environment] Error on TestFTP ' + err.toString());
                        reject('SERVER_ERROR_FTP_NOT_CLIENT');
                    });
                }
                else {
                    reject('SERVER_ERROR_FTP_NOT_CLIENT');
                }
            }
            catch (err) {
                reject(err);
            }
        });
    }
    function TestS3(config) {
        let awsS3 = new aws_s3_implementation_1.AwsS3Implementation(config.region, config.accessKeyId, config.secretAccessKey, config.bucket);
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return awsS3.TestBucket(config.bucket);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_S3_BUCKET_NOT_FOUND';
            return awsS3.TestPath(config.bucket, config.path);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_S3_PATH_NOT_FOUND';
            return Promise.resolve({ status: true, code: 'SERVER_SUCCESS_S3_CONNECTION' });
        })
            .catch(err => {
            return Promise.reject(err);
        });
    }
    app.get('/api/environment/get_amazon_regions', (req, res, next) => {
        let awsRegios = [
            new aws_model_1.AwsRegion('us-east-1', 'AMAZON_REGION_US_EAST_1'),
            new aws_model_1.AwsRegion('us-east-2', 'AMAZON_REGION_US_EAST_2')
        ];
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: awsRegios });
        })
            .catch(err => {
            let environmentMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EnvironmentService::get_amazon_regions] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: environmentMsg, error: errorMsg });
        });
    });
}
exports.EnvironmentService = EnvironmentService;
function SaveEnvironment(vcc, environment) {
    let promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveEnvironment(vcc, environment.name, environment.type, environment.description, environment.bucket, environment.ftpDirectory, environment.apiHost, environment.apiPort, environment.apiUser, (environment.apiPassword === '******' ? null : environment.apiPassword), environment.dbDisabled, environment.dbServer, environment.dbPort, environment.dbUser, (environment.dbPassword === '******' ? null : environment.dbPassword), environment.pbxDisabled, environment.pbxServer, environment.pbxPort, environment.pbxUser, (environment.pbxPassword === '******' ? null : environment.pbxPassword), environment.createdDate, environment.createdByUserId, environment.lastModifiedDate, environment.lastModifiedByUserId));
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
function SaveEnvironments(vcc, environments) {
    let promiseArray = [];
    if (environments) {
        environments.forEach(environment => {
            promiseArray.push(mssql_service_1.MSSql.SaveEnvironment(vcc, environment.name, environment.type, environment.description, environment.ftpDirectory, environment.apiHost, environment.apiPort, environment.apiUser, (environment.apiPassword === '******' ? null : environment.apiPassword), environment.dbDisabled, environment.dbServer, environment.dbPort, environment.dbUser, (environment.dbPassword === '******' ? null : environment.dbPassword), environment.pbxDisabled, environment.pbxServer, environment.pbxPort, environment.pbxUser, (environment.pbxPassword === '******' ? null : environment.pbxPassword), environment.createdDate, environment.createdByUserId, environment.lastModifiedDate, environment.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(results => {
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveEnvironments = SaveEnvironments;
