"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CSVBatch = void 0;
const log4js = require("log4js");
let logger = log4js.getLogger('ServerScripts');
class CSVBatch {
    constructor(csv, isReprocess) {
        this.headerTemplateCSV = ['ID', 'Channel', 'Number', 'Gender', 'SilenceTime', 'DurationTime', 'Category1', 'Category2', 'Category3', 'Description', 'Campaign', 'Agent', 'Direction', 'StartDate', 'EndDate', 'DispositionCode', 'RelativePath'];
        this.csv = null;
        this.isReprocess = false;
        this.csv = csv;
        this.isReprocess = isReprocess;
    }
    validateCSV() {
        let logLabel = `[${CSVBatch.name}::${this.validateCSV.name}] `;
        logger.info(`${logLabel}Getting csv lines: ${this.csv.length}`);
        let lines = null;
        if (this.csv.includes('\r\n'))
            lines = this.csv.split('\r\n');
        else if (this.csv.includes('\r'))
            lines = this.csv.split('\r');
        else if (this.csv.includes('\n'))
            lines = this.csv.split('\n');
        else
            lines = [];
        if (lines.length === 0)
            throw `SERVER_ERROR_CSV_CONTENT_EMPTY`;
        let linesLen = lines.length;
        let result = [];
        let headers = lines[0].split(',');
        let headersLen = headers.length;
        let i = null;
        let j = null;
        let obj = {};
        let currentline = [];
        logger.info(`${logLabel}Getting lines: ${linesLen} headers:`, headers, 'headersLength:', headers.length);
        this.validateCSVHeaders(headers);
        for (i = 1; i < linesLen; i++) {
            obj = {};
            currentline = lines[i].split(',');
            if (currentline?.length > 1) {
                for (j = 0; j <= headersLen; j++) {
                    if (j === headersLen)
                        obj["LN"] = i + 1;
                    else
                        obj[headers[j]] = currentline[j];
                }
                result.push(obj);
            }
        }
        logger.info(`${logLabel}Getting result lines: ${result.length}`);
        let newResult = this.validateCSVContent(result, this.isReprocess);
        return JSON.stringify(newResult);
    }
    validateCSVHeaders(csvHeaders) {
        let csvTemplateHeaders = this.headerTemplateCSV;
        csvTemplateHeaders.forEach(cth => {
            if (!csvHeaders.includes(cth))
                throw `SERVER_ERROR_CSV_HEADER_${cth.toUpperCase()}_NOT_FOUND`;
        });
    }
    validateCSVContent(csvContent, isReprocess) {
        let csvErrors = this.getErrorsCSVContent(csvContent);
        if (csvErrors.length === 0)
            return csvContent;
        else {
            if (csvErrors.length / csvContent.length > 0.4)
                throw "csv_error_limit_exceededcsvErrors" + this.generateErrorsCSVFileContent(csvErrors);
            else {
                if (isReprocess) {
                    let csvContentWithoutErrors = csvContent.filter(objFromCsvContent => {
                        return !csvErrors.find(objFromCsvErrors => {
                            return objFromCsvContent.ID === objFromCsvErrors.ID;
                        });
                    });
                    return csvContentWithoutErrors;
                }
                else
                    throw "csv_error_limit_acceptablecsvErrors" + csvErrors.length + "csvErrors" + csvContent.length + "csvErrors" + this.generateErrorsCSVFileContent(csvErrors);
            }
        }
    }
    getErrorsCSVContent(csvContent) {
        let csvErrorsArray = [];
        let csvDuplicates = this.getDuplicatesIDs(csvContent);
        for (let cc of csvContent) {
            let errorMessage = "";
            let isDuplicate = csvDuplicates.findIndex(x => x.ID === cc.ID);
            if (isDuplicate >= 0)
                errorMessage += 'Duplicate value' + '/';
            if (cc.ID.length > 100)
                errorMessage += `The character length for the ID exceeds the set limit of 100` + '/';
            if (cc.ID === '')
                errorMessage += `The ID can not be empty` + '/';
            let hasWhiteSpace = (s) => (/\s/).test(s);
            if (hasWhiteSpace(cc.ID))
                errorMessage += `The ID does not accept white spaces` + '/';
            if (cc.ID.startsWith("."))
                errorMessage += `The ID cannot start with the period character (.)` + '/';
            if (cc.ID.includes("#"))
                errorMessage += `The caracter # is not allowed into ID column` + '/';
            if (!['CALL', 'WHATSAPP'].includes(cc.Channel))
                errorMessage += `The Channel is not supported`;
            if (cc.Number.length > 50)
                errorMessage += `The character length for Number exceeds the set limit of 50` + '/';
            let isNotIntegerValue = (s) => !(/^[0-9]+$/).test(s);
            if (cc.DurationTime === '')
                errorMessage += `The Duration can not be empty` + '/';
            if (isNotIntegerValue(cc.DurationTime))
                errorMessage += 'The Duration must be integer' + '/';
            if (cc.Gender.length > 50)
                errorMessage += `The character length for Gender exceeds the set limit of 50.` + '/';
            if (cc.Gender !== '') {
                if (!["F", "M"].includes(cc.Gender))
                    errorMessage += `You have entered an incorrect value for the Gender field (Valid F and M)` + '/';
            }
            if (cc.Category1.length > 100)
                errorMessage += `The character length for Category1 exceeds the set limit of 100.` + '/';
            if (cc.Category2.length > 100)
                errorMessage += `The character length for Category2 exceeds the set limit of 100.` + '/';
            if (cc.Category3.length > 100)
                errorMessage += `The character length for Category3 exceeds the set limit of 100.` + '/';
            if (cc.Description.length > 100)
                errorMessage += `The Character length for Description exceeds the set limit of 100` + '/';
            if (cc.Campaign.length > 100)
                errorMessage += `The character length for Campaign exceeds the set limit of 100` + '/';
            if (cc.Campaign === '')
                errorMessage += `The Campaign can not be empty` + '/';
            if (cc.Agent.length > 50)
                errorMessage += `The character length for the Agent exceeds the set limit of 50` + '/';
            if (cc.Agent === '')
                errorMessage += `The Agent can not be empty` + '/';
            if (hasWhiteSpace(cc.Agent))
                errorMessage += `The Agent does not accept white spaces` + '/';
            if (cc.Direction.length > 1)
                errorMessage += `The character length for Direction exceeds the set limit of 1` + '/';
            if (!["E", "S"].includes(cc.Direction))
                errorMessage += `You have entered an incorrect value for the Direction field (Valid E and S)` + '/';
            if (cc.Direction === '')
                errorMessage += `The Direction can not be empty` + '/';
            if (cc.StartDate === '')
                errorMessage += `The StartDate can not be empty` + '/';
            let invalidCsvFormatDate = (s) => !((/^\d{4}[\-\/\s]?((((0[13578])|(1[02]))[\-\/\s]?(([0-2][0-9])|(3[01])))|(((0[469])|(11))[\-\/\s]?(([0-2][0-9])|(30)))|(02[\-\/\s]?[0-2][0-9]))\s(0[0-9]|1[0-9]|2[0-3]):([0-5][0-9]):([0-5][0-9])/).test(s) || (/^\d{4}[\-\/\s]?((((0[13578])|(1[02]))[\-\/\s]?(([0-2][0-9])|(3[01])))|(((0[469])|(11))[\-\/\s]?(([0-2][0-9])|(30)))|(02[\-\/\s]?[0-2][0-9]))\s(0[0-9]|1[0-9]|2[0-3]):([0-5][0-9])/).test(s));
            if (invalidCsvFormatDate(cc.StartDate) && cc.StartDate !== '')
                errorMessage += `Invalid date format for StartDate` + '/';
            if (cc.EndDate === '')
                errorMessage += `The EndDate can not be empty` + '/';
            if (invalidCsvFormatDate(cc.EndDate) && cc.EndDate !== '')
                errorMessage += `Invalid date format for EndDate` + '/';
            if (cc.DispositionCode.length > 50)
                errorMessage += `The character length for DispositionCode exceeds the set limit of 50` + '/';
            if (cc.RelativePath.length > 500)
                errorMessage += `The character length for Relative path exceeds the set limit of 500` + '/';
            let invalidRelativePath = (s) => !(/^[A-Za-z0-9_{}.-\s|\/-]*$/).test(s);
            if (invalidRelativePath(cc.RelativePath))
                errorMessage += `RelativePath only accept letters,numbers and "/" , "-" , "_"` + '/';
            if (errorMessage !== '') {
                let csvError = { LN: 0, ID: "", errorMessage: "" };
                csvError.ID = cc.ID;
                csvError.errorMessage = errorMessage;
                csvError.LN = cc.LN;
                csvErrorsArray.push(csvError);
            }
        }
        return csvErrorsArray;
    }
    getDuplicatesIDs(csvContent) {
        let IDs = csvContent.map(i => i.ID);
        let sorted = IDs.sort();
        let duplicates = sorted.filter((value, index) => {
            return value === sorted[index + 1] && value !== "";
        });
        return duplicates.map((current => Object.assign({ ID: current })));
    }
    generateCSVFileContent() {
        let headers = this.headerTemplateCSV;
        let result = '';
        if (headers?.length) {
            result = headers.join(',');
            result += '\r\n';
        }
        return result;
    }
    generateErrorsCSVFileContent(data) {
        let headers = ["LN", "ID", "errorMessage"];
        let result = '';
        const replacer = (key, value) => (value === null ? '' : value);
        const csv = data.map((row) => headers
            .map((ID) => JSON.stringify(row[ID], replacer))
            .join(','));
        csv.unshift(headers.join(','));
        const csvArray = csv.join('\r\n');
        return csvArray;
    }
}
exports.CSVBatch = CSVBatch;
